import java.awt.*;
import javax.swing.event.MouseInputAdapter;
import java.awt.event.MouseEvent;

/**
 * MyCanvas: an example subclass of Canvas.
 * In this version, the box is stored as a variable in its own right, rather than
 * being re-created each time.  There should be no change in behavior, except perhaps
 * saving a few microseconds.  We also repaint only the old and new locations of the box,
 * rather than the entire Canvas; this again saves a few microseconds.
 * 
 * @author Stephen Bloch
 * @version 23, Feb 15 2005
 * @since version 19
 */
public class MyCanvas extends Canvas
{
    private String currentString;
    private int boxX = 20, boxY = 10;
    private final int boxWidth = 30, boxHeight = 20;
    private Rectangle box = new Rectangle (boxX, boxY, boxWidth, boxHeight);
    private int stringX = 5, stringY = 15;
    /**
     * Constructor for objects of class MyCanvas
     */
    public MyCanvas()
    {
        this.setSize (200, 300); // default is (0,0), i.e. invisible
        this.currentString = "hello";
        this.addMouseListener (new MouseInputAdapter () {
                public void mouseClicked (MouseEvent evt) {
                    repaint (box.x, box.y, box.width+1, box.height+1);
                    box.setLocation(evt.getX(), evt.getY());
                    repaint (box.x, box.y, box.width+1, box.height+1);
                    // the "+1"'s are to make sure the bottom and right edges
                    // get repainted
                }});
    }

    /**
     * @see java.awt.Canvas.paint()
     */
    public void paint (Graphics g) {
        // In Java 1.2 and above, the Graphics object is really a Graphics2D object,
        // which allows you to do more cool stuff.  So we'll convert it.
        Graphics2D g2 = (Graphics2D)g;
        
        g2.draw (this.box);
        
        g2.setPaint (Color.blue);
        g2.drawString (this.currentString, this.stringX, this.stringY);
        
        int[] xpoints = { 50, 100, 100, 70, 62 };
        int[] ypoints = { 100, 70, 145, 140, 70 };
        Polygon shape = new Polygon (xpoints, ypoints, 5);
        
        g2.setPaint (Color.magenta);
        g2.fill (shape);
    }

    /**
     * set the String currently being displayed in the Canvas.
     * @param newString the new String to display.
     * @since version 20
     */
    public void setCurrentString (String newString) {
        this.currentString = newString;
        this.repaint();
        // If you leave this out, system doesn't know that anything
        // has changed, so it doesn't bother calling paint().
    }

}